@@ -4,7 +4,7 @@ Requires the PuLP library.
 """
 
 import pulp
-import pydecode.hyper as ph
+import pydecode
 from collections import defaultdict
 
 
@@ -86,7 +86,7 @@ class HypergraphLP:
             path_edges = [edge
                           for edge in self.hypergraph.edges
                           if pulp.value(self.edge_vars[edge.id]) == 1.0]
-            return ph.Path(self.hypergraph, path_edges)
+            return pydecode.Path(self.hypergraph, path_edges)
 
     @property
     def objective(self):
@@ -101,8 +101,8 @@ class HypergraphLP:
     def decode_fractional(self):
         vec = [pulp.value(self.edge_vars[edge.id])
                for edge in self.hypergraph.edges]
-        weights = ph.LogViterbiPotentials(self.hypergraph).from_vector(vec)
-        return ph.best_path(self.hypergraph, weights)
+        weights = pydecode.LogViterbiPotentials(self.hypergraph).from_vector(vec)
+        return pydecode.best_path(self.hypergraph, weights)
 
     def add_constraints(self, constraints):
         """
@@ -115,7 +115,7 @@ class HypergraphLP:
         """
         for i, constraint in enumerate(constraints):
             self.lp += 0 == \
-                constraints.bias[i][1] + \
+                constraints.bias[i, 0] + \
                 pulp.lpSum([coeff * self.edge_vars[edge.id]
                             for (coeff, edge) in constraint])
 
@@ -181,13 +181,13 @@ class HypergraphLP:
                 in_edges[node.id].append(edge)
 
         for edge in hypergraph.edges:
-            p = potentials[edge]
+            p = potentials[edge.id]
             v = edge_vars[edge.id]
 
         # max \theta x
         prob += pulp.lpSum(
-            [potentials[edge] * edge_vars[edge.id]
-             for edge in hypergraph.edges]) + potentials.bias
+            [potentials[edge.id] * edge_vars[edge.id]
+             for edge in hypergraph.edges])
 
         # x(r) = 1
         prob += node_vars[hypergraph.root.id] == 1
