@@ -1,7 +1,7 @@
 """
 Optimization algorithms.
 """
-import pydecode.hyper as ph
+import pydecode
 import numpy as np
 from numpy.linalg import norm
 import time
@@ -41,7 +41,7 @@ def best_constrained_path(graph, potentials, constraints):
                                      opt.optimizer.best_value,
                                      opt.optimizer.last_subgradient))
         else:
-            opt.descent_round(1)
+            path = opt.descent_round(1)
         if opt.optimizer.check_optimal(opt.optimizer.last_subgradient):
             break
     return path
@@ -118,7 +118,7 @@ class HypergraphOptimizerWithPrimal:
 class ConstrainedHypergraphSubgradientGenerator:
     def __init__(self, graph, weight_potentials, constraint_potentials):
         self.dual_weights = weight_potentials.clone()
-        self.chart = ph.LogViterbiChart(graph)
+        self.chart = pydecode.LogViterbiChart(graph)
         self.current_graph = graph
         self.constraint_potentials = constraint_potentials
         self.weight_potentials = weight_potentials
@@ -134,18 +134,17 @@ class ConstrainedHypergraphSubgradientGenerator:
         """
 
         if x_diff is None:
-            ph.pairwise_dot(self.constraint_potentials, x, self.dual_weights)
+            pydecode.pairwise_dot(self.constraint_potentials, x, self.dual_weights)
         else:
-            ph.pairwise_dot(self.constraint_potentials, x_diff,
+            pydecode.pairwise_dot(self.constraint_potentials, x_diff,
                             self.dual_weights)
 
-        path = ph.best_path(self.current_graph, self.dual_weights)
+        path = pydecode.best_path(self.current_graph, self.dual_weights)
         dual_score = self.dual_weights.dot(path)
         constraint_vector = self.constraint_potentials.dot(path)
         subgradient = np.zeros(len(x))
         for i, j in constraint_vector:
             subgradient[i] = j
-
         return path, subgradient, dual_score
 
 
@@ -169,7 +168,7 @@ def polyak(t, f_x, f_x_best, g):
       Current subgradient vector.
     """
     if norm(g * g) > 0:
-        return (f_x - f_x_best + 1.0/(t+1)) / (g * g)
+        return (f_x - f_x_best) / norm(g * g) ** 2
     else:
         return 0.0
 
@@ -308,7 +307,7 @@ class SubgradientHistory:
 class SubgradientGenerator:
     def __init__(self, graph, weight_potentials, potentials):
         self.dual_weights = weight_potentials.clone()
-        self.chart = ph.LogViterbiChart(graph)
+        self.chart = pydecode.LogViterbiChart(graph)
         self.current_graph = graph
         self.pruning_function = None
         self.potentials = potentials
@@ -318,13 +317,13 @@ class SubgradientGenerator:
         status = history.status()
 
         if status.x_diff is None:
-            ph.pairwise_dot(self.potentials, status.x, self.dual_weights)
+            pydecode.pairwise_dot(self.potentials, status.x, self.dual_weights)
         else:
-            ph.pairwise_dot(self.potentials, status.x_diff, self.dual_weights)
+            pydecode.pairwise_dot(self.potentials, status.x_diff, self.dual_weights)
         path = self.dual_weights.kind.viterbi(self.current_graph,
                                               self.dual_weights,
                                               self.chart).path
-        # path = ph.best_path(self.current_graph,
+        # path = pydecode.best_path(self.current_graph,
         #                     self.dual_weights)
 
         score = self.dual_weights.dot(path)
@@ -343,7 +342,7 @@ class SubgradientGenerator:
                                       self.potentials,
                                       history)
 
-            # pruning = ph.prune_hypergraph(self.current_graph,
+            # pruning = pydecode.prune_hypergraph(self.current_graph,
             #                               self.dual_weights,
             #                               history.best_primal)
 
@@ -351,9 +350,9 @@ class SubgradientGenerator:
                 graph, dual_weights, potentials = pruning
                 self.dual_weights = dual_weights
                 self.potentials = potentials
-                print "OLD SIZE", len(self.current_graph.nodes)
+                print "OLD SIZE", len(self.current_grapydecode.nodes)
                 self.current_graph = graph
-                self.chart = ph.LogViterbiChart(self.current_graph)
+                self.chart = pydecode.LogViterbiChart(self.current_graph)
                 print "NEW SIZE", len(self.current_graph.nodes)
                 prune = history.gap()
 
@@ -367,7 +366,7 @@ class SubgradientGenerator:
 
 
 def _subgradient(graph, weight_potentials, potentials,
-                 best_path_fn=ph.best_path):
+                 best_path_fn=pydecode.best_path):
     r"""
     Compute a subgradient with respect to potentials.
 
