@@ -10,9 +10,8 @@
 #include <utility>
 #include <vector>
 
-#include "Hypergraph/Algorithms.h"
-#include "Hypergraph/Potentials.h"
-
+#include "Hypergraph/Algorithms.hh"
+#include "Hypergraph/Potentials.hh"
 
 using namespace std;
 
@@ -41,42 +40,6 @@ bool is_disjoint(const Set1 &set1, const Set2 &set2) {
     return true;
 }
 
-
-template<typename StatSem>
-struct NodeScore {
-    NodeScore()
-            :
-            count(-1),
-            edge(NULL),
-            back(0),
-            score(StatSem::zero()) {}
-
-    NodeScore(int _count, HEdge _edge,
-              typename StatSem::ValType _score)
-            :
-            count(_count),
-            edge(_edge),
-            back(0),
-            score(_score) {}
-
-    NodeScore(int _count, HEdge _edge, int i, int j,
-              typename StatSem::ValType _score)
-            :
-            count(_count),
-            edge(_edge),
-            back(2),
-            score(_score) {
-        back[0] = i;
-        back[1] = j;
-    }
-
-    int count;
-    HEdge edge;
-    vector<int> back;
-    typename StatSem::ValType score;
-};
-
-
 struct EdgeGroup {
     EdgeGroup(HEdge _edge, int back_left, int back_right)
             : edge(_edge), back(2) {
@@ -87,6 +50,148 @@ struct EdgeGroup {
     vector<int> back;
 };
 
+
+// Implementation of the Bar-Hillel algorithm.
+HypergraphMap *extend_with_dfa(
+    Hypergraph *hypergraph,
+    const HypergraphPotentials<CountingPotential> &potentials,
+    const DFA &dfa,
+    vector<DFANode> *labels) {
+
+    Hypergraph *new_graph = new Hypergraph(hypergraph->is_unary());
+    vector<vector<DFANode> > new_nodes(
+        hypergraph->nodes().size());
+    vector<vector<vector<DFANode> > > new_indexed(
+        hypergraph->nodes().size());
+
+    vector<DFANode> possible_labels;
+
+    foreach (HNode node, hypergraph->nodes()) {
+        new_indexed[node->id()].resize(dfa.states().size());
+    }
+
+    vector<vector<HNode> > reverse_node_map(
+        hypergraph->nodes().size());
+    vector<vector<HEdge> > reverse_edge_map(
+        hypergraph->edges().size());
+
+    foreach (HNode node, hypergraph->nodes()) {
+        if (node->terminal()) {
+            foreach (int state, dfa.states()) {
+                HNode tmp_node = new_graph->add_terminal_node();
+                reverse_node_map[node->id()].push_back(tmp_node);
+                int id = new_nodes[node->id()].size();
+                DFANode label(state, state, tmp_node, id);
+                possible_labels.push_back(label);
+                new_nodes[node->id()].push_back(label);
+                new_indexed[node->id()][state].push_back(label);
+            }
+        } else {
+            set<int> lefts;
+            set<int> rights;
+            vector<vector<vector<EdgeGroup> > > hyps(dfa.states().size());
+            foreach (int state, dfa.states()) {
+                hyps[state].resize(dfa.states().size());
+            }
+            foreach (HEdge edge, node->edges()) {
+                bool unary = hypergraph->tail_nodes(edge) == 1;
+                HNode left_node = hypergraph->tail_node(edge, 0);
+                vector<DFANode> &base = new_nodes[left_node->id()];
+                int symbol = potentials.score(edge);
+                for (int i = 0; i < base.size(); ++i) {
+                    if (unary) {
+                        if (!dfa.valid_transition(base[i].right_state,
+                                                  symbol)) {
+                            continue;
+                        }
+
+                        int right_state =
+                                dfa.transition(base[i].right_state,
+                                               symbol);
+
+                        hyps[base[i].left_state][right_state].push_back(
+                            EdgeGroup(edge, i, 0));
+                        lefts.insert(base[i].left_state);
+                        rights.insert(right_state);
+
+                    } else {
+                        HNode right_node = hypergraph->tail_node(edge, 1);
+                        vector<DFANode> &base_right =
+                                new_indexed[right_node->id()][base[i].right_state];
+                        for (int j = 0; j < base_right.size(); ++j) {
+                            if (!dfa.valid_transition(base_right[j].right_state,
+                                                      symbol)) {
+                                continue;
+                            }
+                            int right_state =
+                                    dfa.transition(base_right[j].right_state,
+                                                   symbol);
+                            hyps[base[i].left_state][right_state].push_back(
+                                EdgeGroup(edge, i, base_right[j].id));
+
+                            lefts.insert(base[i].left_state);
+                            rights.insert(right_state);
+                        }
+                    }
+                }
+            }
+
+            // Make new decorated nodes.
+            for (set<int>::iterator iter = lefts.begin();
+                 iter != lefts.end(); ++iter) {
+                int state_left = *iter;
+                for (set<int>::iterator iter2 = rights.begin();
+                     iter2 != rights.end(); ++iter2) {
+                    int state_right = *iter2;
+                    if (hyps[state_left][state_right].size() == 0) continue;
+                    if (hypergraph->root()->id() == node->id() &&
+                        !(state_left == 0 &&
+                          dfa.final(state_right))) continue;
+
+                    HNode tmp_node = new_graph->start_node();
+                    int id = new_nodes[node->id()].size();
+                    DFANode label(state_left, state_right,
+                                  tmp_node, id);
+                    new_nodes[node->id()].push_back(label);
+                    new_indexed[node->id()][state_left].push_back(label);
+
+                    foreach (EdgeGroup edge_group, hyps[state_left][state_right]) {
+                        HEdge edge = edge_group.edge;
+                        vector<HNode> tails;
+                        for (int i = 0; i < hypergraph->tail_nodes(edge); ++i) {
+                            tails.push_back(new_nodes[hypergraph->tail_node(edge, i)->id()]
+                                            [edge_group.back[i]].node);
+                        }
+                        HEdge new_edge = new_graph->add_edge(tails);
+                        reverse_edge_map[hypergraph->id(edge)].push_back(new_edge);
+                    }
+                    if (new_graph->end_node()) {
+                        reverse_node_map[node->id()].push_back(
+                            new_nodes[node->id()].back().node);
+                        possible_labels.push_back(label);
+                    }
+                }
+            }
+        }
+    }
+    if (reverse_node_map[hypergraph->root()->id()].size() != 1) {
+        throw HypergraphException("New hypergraph has root size is not 1.");
+    }
+    new_graph->finish();
+
+
+    foreach (DFANode label, possible_labels) {
+        if (label.node != NULL && label.node->id() >= 0) {
+            labels->push_back(label);
+        }
+    }
+
+    return HypergraphMap::make_reverse_map(
+        new_graph, hypergraph,
+        reverse_node_map, reverse_edge_map);
+}
+
+
 struct NodeCount {
     NodeCount(int _count, HNode _node) {
         count = _count;
@@ -96,7 +201,6 @@ struct NodeCount {
     HNode node;
 };
 
-
 HypergraphMap *extend_hypergraph_by_count(
     Hypergraph *hypergraph,
     const HypergraphPotentials<CountingPotential> &potentials,
@@ -107,12 +211,11 @@ HypergraphMap *extend_hypergraph_by_count(
     int limit = upper_limit - lower_limit;
     int modified_goal = goal - lower_limit;
 
-    Hypergraph *new_graph = new Hypergraph();
+    Hypergraph *new_graph = new Hypergraph(hypergraph->is_unary());
+    vector<vector<HNode> > reverse_node_map(hypergraph->nodes().size());
+    vector<vector<HEdge> > reverse_edge_map(hypergraph->edges().size());
     vector<vector<NodeCount > > new_nodes(hypergraph->nodes().size());
 
-    vector<vector<HNode> > reverse_node_map(hypergraph->nodes().size());;
-    vector<vector<HEdge> > reverse_edge_map(hypergraph->edges().size());;
-
     foreach (HNode node, hypergraph->nodes()) {
         if (node->terminal()) {
             // The node is a terminal, so just add it.
@@ -123,9 +226,9 @@ HypergraphMap *extend_hypergraph_by_count(
             // Bucket edges.
             vector<vector<EdgeGroup> > counts(limit + 1);
             foreach (HEdge edge, node->edges()) {
-                bool unary = edge->tail_nodes().size() == 1;
-                assert(edge->tail_nodes().size() <= 2);
-                HNode left_node = edge->tail_nodes()[0];
+                bool unary = hypergraph->tail_nodes(edge) == 1;
+                assert(hypergraph->tail_nodes(edge) <= 2);
+                HNode left_node = hypergraph->tail_node(edge, 0);
 
                 int score = potentials.score(edge);
                 vector<NodeCount> &base = new_nodes[left_node->id()];
@@ -138,7 +241,7 @@ HypergraphMap *extend_hypergraph_by_count(
                         counts[total - lower_limit].push_back(
                             EdgeGroup(edge, i, 0));
                     } else {
-                        HNode right_node = edge->tail_nodes()[1];
+                        HNode right_node = hypergraph->tail_node(edge, 1);
                         int right_size = new_nodes[right_node->id()].size();
                         for (int j = 0; j < right_size; ++j) {
                             int total =
@@ -166,12 +269,12 @@ HypergraphMap *extend_hypergraph_by_count(
                 foreach (EdgeGroup edge_group, counts[count]) {
                     HEdge edge = edge_group.edge;
                     vector<HNode> tails;
-                    for (int i = 0; i < edge->tail_nodes().size(); ++i) {
-                        tails.push_back(new_nodes[edge->tail_nodes()[i]->id()]
+                    for (int i = 0; i < hypergraph->tail_nodes(edge); ++i) {
+                        tails.push_back(new_nodes[hypergraph->tail_node(edge, i)->id()]
                                         [edge_group.back[i]].node);
                     }
                     HEdge new_edge = new_graph->add_edge(tails);
-                    reverse_edge_map[edge->id()].push_back(new_edge);
+                    reverse_edge_map[hypergraph->id(edge)].push_back(new_edge);
                 }
                 if (new_graph->end_node()) {
                     reverse_node_map[node->id()].push_back(
@@ -182,54 +285,38 @@ HypergraphMap *extend_hypergraph_by_count(
     }
     new_graph->finish();
 
-    // Create node maps.
-    vector<HNode> *node_map =
-            new vector<HNode>(new_graph->nodes().size(), NULL);
-    vector<HEdge> *edge_map =
-            new vector<HEdge>(new_graph->edges().size(), NULL);
-
-
-    foreach (HNode node, hypergraph->nodes()) {
-        foreach (HNode new_node, reverse_node_map[node->id()]) {
-            if (new_node->id() == -1) continue;
-            (*node_map)[new_node->id()] = node;
-        }
-    }
-    foreach (HEdge edge, hypergraph->edges()) {
-        foreach (HEdge new_edge, reverse_edge_map[edge->id()]) {
-            if (new_edge->id() == -1) continue;
-            (*edge_map)[new_edge->id()] = edge;
-        }
-    }
-    return new HypergraphMap(new_graph, hypergraph,
-                                    node_map, edge_map, false);
+    return HypergraphMap::make_reverse_map(
+        new_graph, hypergraph,
+        reverse_node_map, reverse_edge_map);
 }
 
-Chart<SetPotential> *edge_domination(const Hypergraph &graph) {
-    vector<set<int> > v(graph.edges().size());
-    foreach (HEdge edge, graph.edges()) {
-        v[edge->id()].insert(edge->id());
-    }
 
-    HypergraphVectorPotentials<SetPotential> potentials(
-        &graph, v, set<int>());
-    Chart<SetPotential> *chart = general_inside(&graph, potentials);
-    return chart;
-}
+// Chart<SetPotential> *edge_domination(const Hypergraph &graph) {
+//     vector<set<int> > *v = new vector<set<int> >(graph.edges().size());
+//     foreach (HEdge edge, graph.edges()) {
+//         (*v)[graph.id(edge)].insert(graph.id(edge));
+//     }
 
-Chart<SetPotential> *node_domination(const Hypergraph &graph) {
-    vector<set<int> > v(graph.edges().size());
-    foreach (HEdge edge, graph.edges()) {
-        foreach (HNode node, edge->tail_nodes()) {
-            v[edge->id()].insert(node->id());
-        }
-    }
+//     HypergraphVectorPotentials<SetPotential> potentials(
+//         &graph, v, false /* copy */);
+//     Chart<SetPotential> *chart = general_inside(&graph, potentials);
+//     return chart;
+// }
 
-    HypergraphVectorPotentials<SetPotential> potentials(
-        &graph, v, set<int>());
-    Chart<SetPotential> *chart = general_inside(&graph, potentials);
-    return chart;
-}
+// Chart<SetPotential> *node_domination(const Hypergraph &graph) {
+//     vector<set<int> > *v = new vector<set<int> >(graph.edges().size());
+//     foreach (HEdge edge, graph.edges()) {
+//         for (int i = 0; i < graph.tail_nodes(edge); ++i) {
+//             HNode node = graph.tail_node(edge, i);
+//             (*v)[graph.id(edge)].insert(node->id());
+//         }
+//     }
+
+//     HypergraphVectorPotentials<SetPotential> potentials(
+//         &graph, v, false /* copy */);
+//     Chart<SetPotential> *chart = general_inside(&graph, potentials);
+//     return chart;
+// }
 
 vector<set<int> > *children_sparse(
     const Hypergraph *graph,
@@ -266,7 +353,8 @@ vector<set<int> > *children_nodes(const Hypergraph &graph) {
             new vector<set<int> >(graph.nodes().size());
     foreach (HNode node, graph.nodes()) {
         foreach (HEdge edge, node->edges()) {
-            foreach (HNode tail, edge->tail_nodes()) {
+            for (int i = 0; i < graph.tail_nodes(edge); ++i) {
+                HNode tail = graph.tail_node(edge, i);
                 (*children)[node->id()].insert(tail->id());
             }
         }
@@ -282,7 +370,7 @@ HypergraphMap *project_hypergraph(
     vector<HNode> *node_map =
             new vector<HNode>(hypergraph->nodes().size(), NULL);
     vector<HEdge> *edge_map =
-            new vector<HEdge>(hypergraph->edges().size(), NULL);
+            new vector<HEdge>(hypergraph->edges().size(), -1);
 
     Hypergraph *new_graph = new Hypergraph();
     foreach (HNode node, hypergraph->nodes()) {
@@ -298,7 +386,8 @@ HypergraphMap *project_hypergraph(
                 if (!static_cast<bool>(edge_mask[edge])) continue;
                 vector<HNode> tails;
                 bool all_tails_exist = true;
-                foreach (HNode tail_node, edge->tail_nodes()) {
+                for (int i = 0; i < hypergraph->tail_nodes(edge); ++i) {
+                    HNode tail_node =  hypergraph->tail_node(edge, i );
                     HNode new_tail_node = (*node_map)[tail_node->id()];
                     if (new_tail_node == NULL) {
                         // The tail node was pruned.
@@ -310,7 +399,7 @@ HypergraphMap *project_hypergraph(
                 }
                 if (all_tails_exist) {
                     HEdge new_edge = new_graph->add_edge(tails);
-                    (*edge_map)[edge->id()] = new_edge;
+                    (*edge_map)[hypergraph->id(edge)] = new_edge;
                 }
             }
             bool success = true;
@@ -333,7 +422,7 @@ HypergraphMap *binarize(const Hypergraph *hypergraph) {
     vector<HNode> *nodes =
             new vector<HNode>(hypergraph->nodes().size(), NULL);
     vector<HEdge> *edges =
-            new vector<HEdge>(hypergraph->edges().size(), NULL);
+            new vector<HEdge>(hypergraph->edges().size(), -1);
 
     foreach (HNode node, hypergraph->nodes()) {
         if (node->terminal()) {
@@ -343,22 +432,23 @@ HypergraphMap *binarize(const Hypergraph *hypergraph) {
         vector<vector<HNode> > new_edges;
         foreach (HEdge edge, node->edges()) {
             vector<HNode> new_edge(2);
-            if (edge->tail_nodes().size() <= 2) {
+            if (hypergraph->tail_nodes(edge) <= 2) {
                 vector<HNode> new_tail;
-                foreach (HNode tnode, edge->tail_nodes()) {
+                for (int i = 0; i < hypergraph->tail_nodes(edge); ++i) {
+                    HNode tnode = hypergraph->tail_node(edge, i);
                     new_tail.push_back((*nodes)[tnode->id()]);
                 }
                 new_edges.push_back(new_tail);
                 continue;
             }
-            new_edge[0] = (*nodes)[edge->tail_nodes()[0]->id()];
-            new_edge[1] = (*nodes)[edge->tail_nodes()[1]->id()];
+            new_edge[0] = (*nodes)[hypergraph->tail_node(edge, 0)->id()];
+            new_edge[1] = (*nodes)[hypergraph->tail_node(edge, 1)->id()];
             HNode new_node = range_hyper->start_node();
             range_hyper->add_edge(new_edge);
             range_hyper->end_node();
 
-            for (int i = 2; i < edge->tail_nodes().size() - 1; ++i) {
-                HNode tnode = edge->tail_nodes()[i];
+            for (int i = 2; i < hypergraph->tail_nodes(edge) - 1; ++i) {
+                HNode tnode = hypergraph->tail_node(edge, i);
                 HNode tmp = range_hyper->start_node();
                 new_edge[0] = new_node;
                 new_edge[1] = (*nodes)[tnode->id()];
@@ -367,15 +457,14 @@ HypergraphMap *binarize(const Hypergraph *hypergraph) {
                 new_node = tmp;
             }
             new_edge[0] = new_node;
-            new_edge[1] = (*nodes)[edge->tail_nodes().back()->id()];
+            new_edge[1] = (*nodes)[hypergraph->tail_node(edge, hypergraph->tail_nodes(edge) - 1)->id()];
             new_edges.push_back(new_edge);
         }
         (*nodes)[node->id()] = range_hyper->start_node();
 
         assert(new_edges.size() == node->edges().size());
-        for (int i = 0; i < new_edges.size(); ++i) {
-            (*edges)[node->edges()[i]->id()] =
-                    range_hyper->add_edge(new_edges[i]);
+        for (uint i = 0; i < new_edges.size(); ++i) {
+            (*edges)[i] = range_hyper->add_edge(new_edges[i]);
         }
         range_hyper->end_node();
     }
@@ -383,6 +472,46 @@ HypergraphMap *binarize(const Hypergraph *hypergraph) {
                              nodes, edges, true);
 }
 
+
+Hypergraph *make_lattice(int width, int height,
+                         const vector<vector<int> > &transitions,
+                         vector<LatticeLabel> *labels) {
+    Hypergraph *graph = new Hypergraph(true);
+    HNode source = graph->add_terminal_node();
+    labels->push_back(LatticeLabel(0, 0));
+
+    vector<HNode> old_nodes(height), new_nodes(height);
+    for (int j = 0; j < height; ++j) {
+        old_nodes[j] = graph->start_node();
+        labels->push_back(LatticeLabel(1, j));
+        graph->add_edge(source);
+        graph->end_node();
+    }
+
+    for (int i = 1; i < width; ++i) {
+        for (int j = 0; j < height; ++j) {
+            new_nodes[j] = graph->start_node();
+            labels->push_back(LatticeLabel(i + 1, j));
+            foreach (int k, transitions[j]) {
+                graph->add_edge(old_nodes[k]);
+            }
+            graph->end_node();
+        }
+        old_nodes = new_nodes;
+    }
+
+    graph->start_node();
+    labels->push_back(LatticeLabel(width + 1, 0));
+    for (int j = 0; j < height; ++j) {
+        graph->add_edge(old_nodes[j]);
+    }
+    graph->end_node();
+
+    graph->finish();
+    return graph;
+}
+
+
 // vector<bool> active_nodes(const Hypergraph *graph, set<int> on_edges) {
 //     Chart<SetPotential> *edges = edge_domination;
 
